package com.example.demo.department.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.configuration.Constants;
import com.example.demo.department.service.DepartmentService;
import com.example.demo.department.model.DepartmentEntity;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/departments")
public class DepartmentController {
    private final DepartmentService departmentService;
    private final ModelMapper modelMapper;

    public DepartmentController(DepartmentService departmentService, ModelMapper modelMapper) {
        this.departmentService = departmentService;
        this.modelMapper = modelMapper;
    }

    private DepartmentDto toDto(DepartmentEntity entity) {
        return modelMapper.map(entity, DepartmentDto.class);
    }

    private DepartmentEntity toEntity(DepartmentDto dto) {
        return modelMapper.map(dto, DepartmentEntity.class);
    }

    @GetMapping
    public List<DepartmentDto> getAll() {
        return departmentService.getAll().stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public DepartmentDto get(@PathVariable(name = "id") Long id) {
        return toDto(departmentService.get(id));
    }

    @PostMapping
    public DepartmentDto create(@RequestBody @Valid DepartmentDto dto) {
        return toDto(departmentService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public DepartmentDto update(@PathVariable(name = "id") Long id, @RequestBody DepartmentDto dto) {
        return toDto(departmentService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public DepartmentDto delete(@PathVariable(name = "id") Long id) {
        return toDto(departmentService.delete(id));
    }
}
