package com.example.demo.news.service;

import java.util.List;
import java.util.Optional;
import java.util.Objects;
import org.springframework.stereotype.Service;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.news.model.NewsEntity;
import com.example.demo.news.repository.NewsRepository;

@Service
public class NewsService {
    private final NewsRepository repository;

    public NewsService(NewsRepository repository) {
        this.repository = repository;
    }

    public List<NewsEntity> getAll(Long departmentId) {
        if (Objects.equals(departmentId, 0L)) {
            return repository.getAll();
        }
        return repository.getAll().stream()
                .filter(item -> item.getDepartment().getId().equals(departmentId))
                .toList();
    }

    public NewsEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public NewsEntity create(NewsEntity entity) {
        return repository.create(entity);
    }

    public NewsEntity update(Long id, NewsEntity entity) {
        final NewsEntity existsEntity = get(id);
        existsEntity.setDate(entity.getDate());
        existsEntity.setName(entity.getName());
        existsEntity.setDescription(entity.getDescription());
        existsEntity.setDepartment(entity.getDepartment());
        return repository.update(existsEntity);
    }

    public NewsEntity delete(Long id) {
        final NewsEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }
}
