package com.example.demo.java;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.department.model.DepartmentEntity;
import com.example.demo.news.model.NewsEntity;
import com.example.demo.news.service.NewsService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class NewsTests {
    @Autowired
    private NewsService newsService;

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> newsService.get(0L));
    }

    @Test
    @Order(1)
    void createTest() {
        newsService.create(new NewsEntity(null, "2023-12-23", "Новость0",
                "УлГТУ000", new DepartmentEntity(null, "ИСТ")));
        newsService.create(new NewsEntity(null, "2023-12-24", "Новость1", "УлГТУ111",
                new DepartmentEntity(null, "ИСТ")));
        final NewsEntity last = newsService.create(new NewsEntity(null, "2023-12-25",
                "Новость2", "УлГТУ222", new DepartmentEntity(null, "Гум")));
        Assertions.assertEquals(3, newsService.getAll(0L).size());
        Assertions.assertEquals(last, newsService.get(3L));
    }

    @Test
    @Order(2)
    void updateTest() {
        final String testDate = "TESTDATE";
        final String testName = "TESTNAME";
        final String testDes = "TESTDES";
        final DepartmentEntity testDep = new DepartmentEntity(null, "TESTDEP");
        final NewsEntity entity = newsService.get(3L);
        final String oldDate = entity.getDate();
        final String oldName = entity.getName();
        final String oldDes = entity.getDescription();
        final DepartmentEntity oldDep = entity.getDepartment();
        final NewsEntity newEntity = newsService.update(3L,
                new NewsEntity(1L, testDate, testName, testDes, testDep));
        Assertions.assertEquals(3, newsService.getAll(0L).size());
        Assertions.assertEquals(newEntity, newsService.get(3L));

        Assertions.assertEquals(testDate, newEntity.getDate());
        Assertions.assertNotEquals(oldDate, newEntity.getDate());

        Assertions.assertEquals(testName, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());

        Assertions.assertEquals(testDes, newEntity.getDescription());
        Assertions.assertNotEquals(oldDes, newEntity.getDescription());

        Assertions.assertEquals(testDep, newEntity.getDepartment());
        Assertions.assertNotEquals(oldDep, newEntity.getDepartment());
    }

    @Test
    @Order(3)
    void deleteTest() {
        newsService.delete(3L);
        Assertions.assertEquals(2, newsService.getAll(0L).size());
        final NewsEntity last = newsService.get(2L);
        Assertions.assertEquals(2L, last.getId());

        final NewsEntity newEntity = newsService
                .create(new NewsEntity(null, "test1", "test2", "test3", new DepartmentEntity(null, "testtt")));
        Assertions.assertEquals(3, newsService.getAll(0L).size());
        Assertions.assertEquals(4L, newEntity.getId());
    }
}
